#pragma once

#include <map>
#include <string>

#include "CoreSegment.h"
#include "detail/EntityContainer.h"

namespace armarx::armem
{

    /**
     * @brief Data of a memory consisting of multiple core segments.
     */
    class Memory : public detail::EntityContainer<CoreSegment, Memory>
    {
        using Base = detail::EntityContainer<CoreSegment, Memory>;

    public:

        Memory();
        Memory(const std::string& name);
        Memory(const MemoryID& id);

        Memory(const Memory& other);
        Memory& operator=(const Memory& other);

        ~Memory() override;


        using Base::name;
        const std::string& name() const override;


        bool hasCoreSegment(const std::string& name) const;

        CoreSegment& getCoreSegment(const std::string& name);
        const CoreSegment& getCoreSegment(const std::string& name) const;


        using Base::getEntity;
        const Entity& getEntity(const MemoryID& id) const override;

        /**
         * @brief Add an empty core segment with the given name.
         * @param name The core segment name.
         * @param coreSegmentType The core segment type (optional).
         * @return The added core segment.
         */
        CoreSegment& addCoreSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr coreSegmentType = nullptr);
        /// Copy and insert a core segment.
        CoreSegment& addCoreSegment(const CoreSegment& coreSegment);
        /// Move and insert a core segment.
        CoreSegment& addCoreSegment(CoreSegment&& coreSegment);

        /**
         * @brief Add multiple core segments.
         * @param The core segment names.
         * @return The core segments. The contained pointers are never null.
         */
        std::vector<CoreSegment*> addCoreSegments(const std::vector<std::string>& names);

        virtual MemoryID update(const EntityUpdate& update) override;
        using Base::update;


        bool equalsDeep(const Memory& other) const;


        // MemoryItem interface
    public:
        std::string getLevelName() const override;

        // MemoryContainer interface
    protected:
        ContainerT& _underlyingContainer() override;
        void _copyElements(Memory& other, bool withData) const override;


    public:
        std::map<std::string, CoreSegment> coreSegments;
    };
}
