#include "Visitor.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/libraries/armem/core/Memory.h>
#include <RobotAPI/libraries/armem/core/error.h>


namespace armarx::armem
{

    Visitor::Visitor()
    {
    }

    Visitor::~Visitor()
    {
    }

    bool Visitor::applyTo(Memory& memory)
    {
        for (auto& [_, coreSeg] : memory)
        {
            if (!applyTo(coreSeg))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(CoreSegment& coreSegment)
    {
        for (auto& [_, provSeg] : coreSegment)
        {
            if (!applyTo(provSeg))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(ProviderSegment& providerSegment)
    {
        for (auto& [_, entity] : providerSegment)
        {
            if (!applyTo(entity))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(Entity& entity)
    {
        for (auto& [_, snapshot] : entity)
        {
            if (!applyTo(snapshot))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(EntitySnapshot& snapshot)
    {
        for (auto& instance : snapshot)
        {
            if (!applyTo(instance))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(EntityInstance& instance)
    {
        return visit(instance);
    }


    bool Visitor::applyTo(const Memory& memory)
    {
        for (const auto& [_, coreSeg] : memory)
        {
            if (!applyTo(coreSeg))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(const CoreSegment& coreSegment)
    {
        for (const auto& [_, provSeg] : coreSegment)
        {
            if (!applyTo(provSeg))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(const ProviderSegment& providerSegment)
    {
        for (const auto& [_, entity] : providerSegment)
        {
            if (!applyTo(entity))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(const Entity& entity)
    {
        for (const auto& [_, snapshot] : entity)
        {
            if (!applyTo(snapshot))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(const EntitySnapshot& snapshot)
    {
        for (const auto& instance : snapshot)
        {
            if (!applyTo(instance))
            {
                return false;
            }
        }
        return true;
    }

    bool Visitor::applyTo(const EntityInstance& instance)
    {
        return visit(instance);
    }

}
