#pragma once

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "../../error.h"

#include "MemoryItem.h"


namespace armarx::armem::detail
{

    /**
     * @class Provides default implmentations of `MemoryContainer`, as well as
     * iterators (which requires a template).
     */
    template <class _ContainerT, class _Derived>
    class MemoryContainer : public MemoryItem
    {
    public:
        using ContainerT = _ContainerT;
        using Derived = _Derived;

    public:

        MemoryContainer()
        {}
        MemoryContainer(const MemoryID& id) : MemoryItem(id)
        {}

        virtual ~MemoryContainer()
        {}


        // Container methods

        virtual bool empty() const
        {
            return _underlyingContainer().empty();
        }
        virtual std::size_t size() const
        {
            return _underlyingContainer().size();
        }
        virtual void clear()
        {
            return _underlyingContainer().clear();
        }

        typename ContainerT::const_iterator begin() const
        {
            return _underlyingContainer().begin();
        }
        typename ContainerT::iterator begin()
        {
            return _underlyingContainer().begin();
        }
        typename ContainerT::const_iterator end() const
        {
            return _underlyingContainer().end();
        }
        typename ContainerT::iterator end()
        {
            return _underlyingContainer().end();
        }


        // Copying

        virtual Derived copy(bool withData = true) const
        {
            auto cast = dynamic_cast<const Derived*>(this);
            ARMARX_CHECK_NOT_NULL(cast);

            Derived other;
            this->_copySelf(other);
            this->_copyElements(other, withData);
            return other;
        }

        /// Make a copy not containing any elements.
        virtual Derived copyEmpty() const
        {
            auto cast = dynamic_cast<const Derived*>(this);
            ARMARX_CHECK_NOT_NULL(cast);

            Derived other;
            this->_copySelf(other);
            return other;
        }


    protected:


        virtual ContainerT& _underlyingContainer() = 0;
        virtual const ContainerT& _underlyingContainer() const
        {
            return const_cast<const ContainerT&>(const_cast<MemoryContainer*>(this)->_underlyingContainer());
        }

        virtual void _copySelf(Derived& other) const
        {
            other.id() = this->id();
            (void) other;
        }
        virtual void _copyElements(Derived& other, bool withData) const
        {
            (void) other, (void) withData;
        }

    };

}
