/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <filesystem>
#include <memory>
#include <string>

#include <RobotAPI/libraries/aron/core/navigator/data/container/Dict.h>
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>
#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>

#include <RobotAPI/libraries/armem/core/EntityInstance.h>

#include "../DiskReaderWriter.h"
#include "../FileSystemLookupMemory.h"


namespace armarx::armem::io
{
    class DiskReader;
    using DiskReaderPtr = std::shared_ptr<DiskReader>;

    class DiskReader : virtual public DiskReaderWriter
    {
    public:

        using DiskReaderWriter::DiskReaderWriter;

        FileSystemLookupMemory readMemoryStructureFromDisk();
        FileSystemLookupMemory readMemoryStructureFromDisk(const std::string&);

        EntityInstance readSingleInstanceFromDisk(const std::filesystem::path&, const aron::typenavigator::NavigatorPtr& expectedStructure = nullptr) const;
        EntityInstance readSingleInstanceFromDisk(const MemoryID&, const aron::typenavigator::NavigatorPtr& expectedStructure = nullptr) const;

        aron::typenavigator::ObjectNavigatorPtr readSingleTypeInformationFromDisk(const std::filesystem::path&) const;
        aron::typenavigator::ObjectNavigatorPtr readSingleTypeInformationFromDisk(const MemoryID&) const;

    protected:
        EntityInstance unwrapData(const aron::datanavigator::DictNavigatorPtr&) const;
        aron::typenavigator::ObjectNavigatorPtr unwrapType(const aron::typenavigator::ObjectNavigatorPtr&) const;

        virtual aron::datanavigator::DictNavigatorPtr getStringAsDataNavigator(const std::string&, const aron::typenavigator::NavigatorPtr&) const = 0;
        virtual aron::typenavigator::ObjectNavigatorPtr getStringAsTypeNavigator(const std::string& s) const = 0;
    };
}
