/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#include "DiskReaderWriter.h"

#include <RobotAPI/libraries/armem/core/error.h>


namespace armarx::armem::io
{

    DiskReaderWriter::DiskReaderWriter(bool createIfNotExistent) :
        DiskReaderWriter("/tmp/MemoryExport", createIfNotExistent)
    {}

    DiskReaderWriter::DiskReaderWriter(const std::filesystem::path& rootPath, bool createIfNotExistent) :
        rootPath(rootPath)
    {
        if (!std::filesystem::exists(rootPath.parent_path()))
        {
            throw error::PathDoesNotExist(rootPath.parent_path().string(),
                                          "The parent path does not exist. At least this directory must exist!");
        }

        if (!std::filesystem::is_directory(rootPath.parent_path()))
        {
            throw error::PathDoesNotExist(rootPath.parent_path().string(), "The parent path is not a directory. It must be a directory.");
        }

        if (!std::filesystem::exists(rootPath))
        {
            if (createIfNotExistent)
            {
                std::filesystem::create_directory(rootPath);
            }
            else
            {
                throw error::PathDoesNotExist(rootPath.string(), "The path does not exist and a new folder should not be created!");
            }
        }

        if (!std::filesystem::is_directory(rootPath))
        {
            throw error::PathNotADirectory(rootPath.string(), "The Path is not valid. It points to a file instead of a folder!");
        }
    }

    DiskReaderWriter::~DiskReaderWriter()
    {
    }

}
