/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <filesystem>
#include <map>
#include <string>
#include <vector>

#include "../Time.h"
#include "../Memory.h"
#include "../error.h"


namespace armarx::armem::io
{
    using DiskEntityInstance = std::filesystem::path; // absolute paths to stored files
    using DiskEntitySnapshot = std::vector<DiskEntityInstance>;
    using DiskEntity = std::map<Time, DiskEntitySnapshot>;
    using DiskProviderSegment = std::map<std::string, DiskEntity>;
    using DiskCoreSegment = std::map<std::string, DiskProviderSegment>;
    using DiskMemory = std::map<std::string, DiskCoreSegment>;


    class FileSystemLookupMemory
    {
    public:
        FileSystemLookupMemory() = default;
        void merge(const FileSystemLookupMemory& m);

        DiskMemory getDiskMemory(const std::string&) const;
        DiskMemory getDiskMemory(const MemoryID&) const;
        DiskCoreSegment getDiskCoreSegment(const MemoryID&) const;
        DiskProviderSegment getDiskProviderSegment(const MemoryID&) const;
        DiskEntity getDiskEntity(const MemoryID&) const;
        DiskEntitySnapshot getDiskEntitySnapshot(const MemoryID&) const;
        DiskEntityInstance getDiskEntityInstance(const MemoryID&) const;

        bool hasMemory(const std::string&) const;
        bool hasMemory(const MemoryID&) const;
        bool hasCoreSegment(const MemoryID&) const;
        bool hasProviderSegment(const MemoryID&) const;
        bool hasEntity(const MemoryID&) const;
        bool hasEntitySnapshot(const MemoryID&) const;
        bool hasEntityInstance(const MemoryID&) const;

        std::map<std::string, DiskMemory> memoryLookupTable;
    };
}
