/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <filesystem>
#include <string>
#include <utility>  // std::pair

// ArmarX
#include <RobotAPI/interface/aron.h>

#include "DiskWriter/DiskWriter.h"
#include "DiskWriter/NlohmannJSONDiskWriter/NlohmannJSONDiskWriter.h"
#include "DiskReader/DiskReader.h"
#include "DiskReader/NlohmannJSONDiskReader/NlohmannJSONDiskReader.h"


namespace armarx::armem::io
{
    class FileSystemMemoryManager
    {
    public:
        FileSystemMemoryManager() = delete;
        FileSystemMemoryManager(const std::filesystem::path& rootPath, bool createFolder = false);
        FileSystemMemoryManager(const io::DiskWriterPtr&, const io::DiskReaderPtr&);

        void writeOnDisk(const Memory& memory);
        void writeOnDisk(const MemoryID& coreSegmentID, const CoreSegment& coreSegment);
        void writeOnDisk(const MemoryID& providerSegmentID, const ProviderSegment& providerSegment);
        void writeOnDisk(const MemoryID& entityID, const Entity& entity);
        // void writeOnDisk(const MemoryID&, const Entity&, aron::typenavigator::ObjectNavigatorPtr = nullptr);
        void writeOnDisk(const MemoryID& entitySnapshotID, const EntitySnapshot& entitySnapshot);

        void update();
        void merge(const FileSystemLookupMemory& info);

        Memory readMemoryFromDisk(const std::string& memoryName) const;
        Memory readMemoryFromDisk(const MemoryID& memoryID) const;
        CoreSegment readCoreSegmentFromDisk(const MemoryID& coreSegmentID) const;
        ProviderSegment readProviderSegmentFromDisk(const MemoryID& providerSegmentID) const;
        Entity readEntityFromDisk(const MemoryID& entityID) const;
        EntitySnapshot readEntitySnapshotFromDisk(const MemoryID& entitySnapshotID) const;
        EntityInstance readEntityInstanceFromDisk(const MemoryID& entityInstanceID) const;

        aron::typenavigator::ObjectNavigatorPtr readClosestTypeInformation(const MemoryID&) const;

        //std::pair<Entity, aron::typenavigator::ObjectNavigatorPtr> readTypedEntityFromDisk();
        //std::pair<EntitySnapshot, aron::typenavigator::ObjectNavigatorPtr> readTypedEntitySnapshotFromDisk();

        std::string toString();

    private:
        FileSystemLookupMemory internalMemory;

        DiskWriterPtr writer;
        DiskReaderPtr reader;
    };
}
