/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/armem/client/Reader.h>

#include "MemoryConnector.h"
#include "TransformInterfaces.h"

namespace armarx::armem
{
    /**
    * @defgroup Component-ExampleClient ExampleClient
    * @ingroup RobotAPI-Components
    * A description of the component ExampleClient.
    *
    * @class ExampleClient
    * @ingroup Component-ExampleClient
    * @brief Brief description of class ExampleClient.
    *
    * Detailed description of class ExampleClient.
    */
    class TransformReader :
        virtual public ::armarx::armem::TransformReaderInterface,
        virtual public ::armarx::armem::MemoryConnector
    {
    public:
        TransformReader(ManagedIceObject& component);

        ~TransformReader() override;

        void connect() override;

        TransformResult getGlobalPose(const std::string& agentName,
                                      const std::string& robotRootFrame,
                                      const std::int64_t& timestamp) const override;
        TransformResult lookupTransform(const TransformQuery& query) const override;

        void registerPropertyDefinitions(::armarx::PropertyDefinitionsPtr& def) override;

        const std::string& getPropertyPrefix() const override
        {
            return propertyPrefix;
        }

    private:
        std::vector<std::string> buildTransformChain(const armem::Memory& memory,
                const TransformQuery& query) const;

        std::vector<Eigen::Affine3f> obtainTransforms(const armem::Memory& memory,
                const std::vector<std::string>& tfChain,
                const std::string& agent, const std::int64_t& timestamp) const;

        Eigen::Affine3f obtainTransform(const std::string& entityName, const armem::ProviderSegment& agentProviderSegment, int64_t timestamp) const;


        armem::client::Reader memoryReader;

        // Properties
        struct Properties
        {
            std::string memoryName             = "RobotState";
            std::string localizationMemoryName = "Localization";
        } properties;


        const std::string propertyPrefix = "mem.localization.read.";
    };
} // namespace armarx::armem
