/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "IntEnum.h"

namespace armarx::aron::cppcodegenerator::serializer
{
    // constructors
    IntEnumSerializer::IntEnumSerializer(const typenavigator::IntEnumNavigatorPtr& n) :
        Serializer(n->getEnumName(), simox::meta::get_type_name(typeid(data::AronNDArray)), simox::meta::get_type_name(typeid(type::AronIntEnum))),
        navigator(n)
    {
    }

    std::vector<CppFieldPtr> IntEnumSerializer::getPublicVariableDeclarations(const std::string& name) const
    {
        CppFieldPtr field = CppFieldPtr(new CppField(getFullCppTypename(), name));
        return {field};
    }

    std::vector<std::pair<std::string, std::string>> IntEnumSerializer::getCtorInitializers(const std::string&) const
    {
        return {};
    }

    CppBlockPtr IntEnumSerializer::getCtorBlock(const std::string&) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        return b;
    }

    CppBlockPtr IntEnumSerializer::getResetBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + ".reset();");
        return b;
    }

    CppBlockPtr IntEnumSerializer::getWriteInitialTypeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + "::writeInitialType(w);");
        return b;
    }

    CppBlockPtr IntEnumSerializer::getWriteCurrentTypeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + ".writeCurrentType(w);");
        return b;
    }

    CppBlockPtr IntEnumSerializer::getInitializeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + ".initialize();");
        return b;
    }

    CppBlockPtr IntEnumSerializer::getWriteBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + ".write(w);");
        return b;
    }

    CppBlockPtr IntEnumSerializer::getReadBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + ".read(r);");
        return b;
    }

    CppBlockPtr IntEnumSerializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))");
        b->addLine("\t return false;");
        return b;
    }
}
