/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <vector>

// ArmarX
#include <RobotAPI/libraries/aron/core/Config.h>
#include <RobotAPI/libraries/aron/core/Descriptor.h>

namespace armarx::aron::dataIO
{
    class ReaderInterface;
    typedef std::shared_ptr<ReaderInterface> ReaderInterfacePtr;

    class ReaderInterface
    {
    public:
        virtual int readStartDict() = 0;
        virtual bool readEndDict() = 0;
        virtual int readStartList() = 0;
        virtual bool readEndList() = 0;

        virtual std::tuple<std::vector<int>, std::string> readStartNDArray() = 0;
        virtual bool readEndNDArray(unsigned char*) = 0;

        virtual void readPrimitive(int&) = 0;
        virtual void readPrimitive(long&) = 0;
        virtual void readPrimitive(float&) = 0;
        virtual void readPrimitive(double&) = 0;
        virtual void readPrimitive(std::string&) = 0;
        virtual void readPrimitive(bool&) = 0;

        template<class T>
        T readPrimitive()
        {
            T p;
            readPrimitive(p);
            return p;
        }

        virtual std::string readKey() = 0;
        virtual void loadMember(const std::string&) = 0;

        // Helper functions
        virtual data::Descriptor getTypeOfNext(const type::Descriptor hint = type::eUnknown) const = 0;
    };
}
