/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/core/io/typeIO/Reader.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/io/typeIO/reader/nlohmannJSON/NlohmannJSONReaderToken.h>

namespace armarx::aron::typeIO::reader
{
    class NlohmannJSONReader;
    typedef std::shared_ptr<NlohmannJSONReader> NlohmannJSONReaderPtr;

    class NlohmannJSONReader :
        virtual public ReaderInterface
    {
    public:
        using PointerType = NlohmannJSONReaderPtr;

    public:
        // constructors
        NlohmannJSONReader() = delete;
        NlohmannJSONReader(const nlohmann::json& n);
        NlohmannJSONReader(const std::string& n);

        virtual std::tuple<std::string, int> readStartObject() override;
        virtual bool readEndObject() override;
        virtual int readStartList() override;
        virtual bool readEndList() override;
        virtual int readStartDict() override;
        virtual bool readEndDict() override;
        virtual int readStartTuple() override;
        virtual bool readEndTuple() override;
        virtual int readStartPair() override;
        virtual bool readEndPair() override;

        virtual std::tuple<std::vector<int>, std::string> readEigenMatrix() override;
        virtual std::string readEigenQuaternion() override;
        virtual std::tuple<unsigned int, unsigned int, std::string> readIVTCByteImage() override;
        virtual std::tuple<std::vector<int>, std::string> readOpenCVMat() override;
        virtual std::tuple<unsigned int, unsigned int, std::string> readPCLPointCloud() override;
        virtual void readPosition() override;
        virtual void readOrientation() override;
        virtual void readPose() override;

        virtual void readInt() override;
        virtual void readLong() override;
        virtual void readFloat() override;
        virtual void readDouble() override;
        virtual void readString() override;
        virtual void readBool() override;
        virtual void readTime() override;

        virtual std::string readKey() override;
        virtual void loadMember(const std::string&) override;

        // Helper functions
        virtual type::Descriptor getTypeOfNext() const override;

    private:
        nlohmann::json getNextAndIncrease();

    private:
        nlohmann::json input;
        bool readInitialStart = false;
        std::stack<NlohmannJSONReaderTokenPtr> stack;
    };
}
