/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/libraries/aron/core/Config.h>
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/core/io/Data.h>

namespace armarx::aron::typeIO
{
    template<typename ElementTypename>
    class WriterTokenInterface;

    template<typename ElementTypename>
    using WriterTokenInterfacePtr = std::shared_ptr<WriterTokenInterface<ElementTypename>>;

    template<typename ElementTypename>
    class WriterTokenInterface
    {
    public:
        using PointerType = WriterTokenInterfacePtr<ElementTypename>;

        // Interface
        virtual void addElement(const ElementTypename&) = 0;
        virtual void setName(const std::string&) = 0;

        // General implementation
        std::string toAccessor() const
        {
            type::Descriptor desc = this->getDescriptor();
            switch (desc)
            {
                case type::Descriptor::eDict:
                case type::Descriptor::eList:
                    return "[ACCEPTED_TYPE]";
                case type::Descriptor::eObject:
                    return this->currentKey;
                case type::Descriptor::eTuple:
                    return std::to_string(this->currentIndex);
                default:
                    throw error::DescriptorNotValidException("NavigatorWriterToken", "toElementAccessor", "Could not resove a type of a navigator. Allowed are only containers.", desc);
            }
        }

        void setCurrentKey(const std::string& s)
        {
            if (s.empty())
            {
                throw error::AronException("AronWriterToken", "setCurrentKey", "The passed key is empty.");
            }
            currentKey = s;
        }

        ElementTypename getElement() const
        {
            return element;
        }

        type::Descriptor getDescriptor() const
        {
            return descriptor;
        }

    protected:
        // members
        type::Descriptor descriptor = type::eUnknown;
        ElementTypename element;

        // current index
        unsigned int currentIndex = 0;

        // current key
        std::string currentKey = "";
    };
}
