/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <map>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/data/Navigator.h>

namespace armarx::aron::datanavigator
{
    class NDArrayNavigator;
    typedef std::shared_ptr<NDArrayNavigator> NDArrayNavigatorPtr;

    class NDArrayNavigator :
        virtual public Navigator
    {
    public:
        using PointerType = NDArrayNavigatorPtr;

    public:
        // constructors
        NDArrayNavigator(const Path& path = Path());
        NDArrayNavigator(const data::AronNDArrayPtr&, const Path& path = Path());
        NDArrayNavigator(const std::vector<int>&, const std::string&, const std::vector<unsigned char>&, const Path& path = Path());

        // operators
        bool operator==(const NDArrayNavigator&) const;
        virtual bool equalsDataNavigator(const NavigatorPtr&) const override;
        bool equalsDeep(const NDArrayNavigatorPtr&) const;

        // static methods
        static NDArrayNavigatorPtr DynamicCast(const NavigatorPtr& n);
        static NDArrayNavigator DynamicCast(Navigator& n);
        static NDArrayNavigatorPtr DynamicCastAndCheck(const NavigatorPtr& n);

        static NDArrayNavigatorPtr FromAronNDArrayPtr(const data::AronNDArrayPtr& aron);
        static data::AronNDArrayPtr ToAronNDArrayPtr(const NDArrayNavigatorPtr& navigator);

        /// Return dimensions in a readable string such as "(2, 3, 4)".
        static std::string DimensionsToString(const std::vector<int>& dimensions);


        // public member functions
        unsigned char* getData() const;
        void setData(unsigned int, const unsigned char*);

        std::vector<int> getDimensions() const;
        void setDimensions(const std::vector<int>&);
        void addDimension(int);

        std::string getType() const;
        void setType(const std::string&);

        data::AronNDArrayPtr toAronNDArrayPtr() const;

        // virtual implementations
        virtual data::AronDataPtr getResult() const override;
        virtual std::string getName() const override;
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;

        virtual typenavigator::NavigatorPtr recalculateType() const override;
        virtual bool fullfillsType(const typenavigator::NavigatorPtr&) const override;

    private:
        data::AronNDArrayPtr aron;
    };
}
