/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <map>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/data/Navigator.h>
#include <RobotAPI/libraries/aron/core/navigator/type/container/Dict.h>

namespace armarx::aron::datanavigator
{
    class DictNavigator;
    typedef std::shared_ptr<DictNavigator> DictNavigatorPtr;

    class DictNavigator :
        virtual public Navigator
    {
    public:
        using PointerType = DictNavigatorPtr;

    public:
        // constructors
        DictNavigator(const Path& path = Path());
        DictNavigator(const data::AronDictPtr&, const Path& path = Path());
        DictNavigator(const data::AronDataDict&, const Path& path = Path());
        DictNavigator(const std::map<std::string, NavigatorPtr>&, const Path& path = Path());

        // operators
        bool operator==(const DictNavigator&) const;
        virtual bool equalsDataNavigator(const NavigatorPtr&) const override;
        bool equalsDeep(const DictNavigatorPtr&) const;

        // static methods
        static DictNavigatorPtr DynamicCast(const NavigatorPtr& n);
        static DictNavigator DynamicCast(Navigator& n);
        static DictNavigatorPtr DynamicCastAndCheck(const NavigatorPtr& n);

        static DictNavigatorPtr FromAronDictPtr(const data::AronDictPtr& aron);
        static data::AronDictPtr ToAronDictPtr(const DictNavigatorPtr& navigator);

        // public member functions
        data::AronDictPtr toAronDictPtr() const;
        std::vector<std::string> getAllKeys() const;

        void addElement(const std::string& key, const NavigatorPtr&);
        bool hasElement(const std::string&) const;
        NavigatorPtr getElement(const std::string&) const;
        std::map<std::string, NavigatorPtr> getElements() const;

        void clear();

        // virtual implementations
        virtual data::AronDataPtr getResult() const override;
        virtual std::string getName() const override;
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;

        virtual typenavigator::NavigatorPtr recalculateType() const override;
        virtual bool fullfillsType(const typenavigator::NavigatorPtr&) const override;

        virtual NavigatorPtr navigateAbsolute(const Path& path) const override;
        virtual NavigatorPtr navigateRelative(const Path& path) const override;

    private:
        // members
        std::map<std::string, NavigatorPtr> childrenNavigators;
        data::AronDictPtr aron;
    };
}
