/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "Navigator.h"

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/type/NavigatorFactory.h>
#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>

namespace armarx::aron::typenavigator
{
    /* Navigator */
    // static data members
    const NavigatorFactoryPtr Navigator::FACTORY = NavigatorFactoryPtr(new NavigatorFactory());

    // constructors
    Navigator::Navigator(const type::Descriptor& d, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(d, path)
    {
    }

    NavigatorPtr Navigator::FromAronType(const type::AronTypePtr& a, const Path& path)
    {
        return FACTORY->create(a, path);
    }

    void Navigator::CheckTypeNavigatorPtrForNull(const std::string& c, const std::string& m, const NavigatorPtr& data)
    {
        if (data.get() == nullptr)
        {
            throw error::AronException(c, m, "Could not cast an NavigatorPtr. The Ptr was NULL.");
        }
    }

    void Navigator::CheckTypeNavigatorPtrForNull(const std::string& c, const std::string& m, const Path& p, const NavigatorPtr& data)
    {
        if (data.get() == nullptr)
        {
            throw error::AronException(c, m, "Could not cast an NavigatorPtr. The Ptr was NULL.", p);
        }
    }

    /* AronSingleAcceptedTypeHavingTypeNavigator */
    /*NavigatorPtr AronSingleAcceptedTypeHavingTypeNavigator::getAcceptedType() const
    {
        return acceptedTypeNavigator;
    }

    size_t AronSingleAcceptedTypeHavingTypeNavigator::childrenSize() const
    {
        return 1;
    }*/

    /* AronMultipleListAcceptedTypeHavingTypeNavigator */
    /*std::vector<NavigatorPtr> AronMultipleListAcceptedTypeHavingTypeNavigator::getAcceptedTypes() const
    {
        return acceptedTypeNavigators;
    }

    NavigatorPtr AronMultipleListAcceptedTypeHavingTypeNavigator::getAcceptedType(unsigned int i) const
    {
        if (i >= acceptedTypeNavigators.size())
        {
            return nullptr;
        }
        return acceptedTypeNavigators[i];
    }

    size_t AronMultipleListAcceptedTypeHavingTypeNavigator::childrenSize() const
    {
        return acceptedTypeNavigators.size();
    }*/

    /* AronMultipleDictAcceptedTypeHavingTypeNavigator */
    /*std::map<std::string, NavigatorPtr> AronMultipleDictAcceptedTypeHavingTypeNavigator::getAcceptedTypes() const
    {
        return acceptedTypeNavigators;
    }

    NavigatorPtr AronMultipleDictAcceptedTypeHavingTypeNavigator::getAcceptedType(const std::string& s) const
    {
        if (acceptedTypeNavigators.find(s) == acceptedTypeNavigators.end())
        {
            return nullptr;
        }
        return acceptedTypeNavigators.at(s);
    }

    size_t AronMultipleDictAcceptedTypeHavingTypeNavigator::childrenSize() const
    {
        return acceptedTypeNavigators.size();
    }

    std::vector<std::string> AronMultipleDictAcceptedTypeHavingTypeNavigator::getAllKeys() const
    {
        std::vector<std::string> ret;
        for (const auto& [key, _] : acceptedTypeNavigators)
        {
            ret.push_back(key);
        }
        return ret;
    }*/

    /* AronComplexTypeNavigator */
    /*size_t AronComplexTypeNavigator::childrenSize() const
    {
        return 0;
    }*/

    /* AronPrimitiveTypeNavigator */
    /*size_t AronPrimitiveTypeNavigator::childrenSize() const
    {
        return 0;
    }*/
}


