/*
* This file is part of ArmarX.
*
* Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
* Karlsruhe Institute of Technology (KIT), all rights reserved.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <vector>
#include <map>
#include <string>

// Simox
#include <SimoxUtility/algorithm/string.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/navigator/Navigator.h>

namespace armarx::aron::typenavigator
{
    class NavigatorFactory;
    typedef std::shared_ptr<NavigatorFactory> NavigatorFactoryPtr;

    class Navigator;
    typedef std::shared_ptr<Navigator> NavigatorPtr;

    class Navigator :
        virtual public aron::Navigator<type::Descriptor, type::AronType>
    {
    public:
        using PointerType = NavigatorPtr;

    public:
        // constructors
        Navigator() = delete;
        Navigator(const type::Descriptor&, const Path&);

        // virtual methods
        virtual std::vector<NavigatorPtr> getChildren() const = 0;
        virtual size_t childrenSize() const = 0;
        virtual type::AronTypePtr getResult() const override = 0;
        virtual std::string getName() const override = 0;

        // static methods
        static NavigatorPtr FromAronType(const type::AronTypePtr&, const Path& = Path());

        template<typename NavigatorType>
        static typename NavigatorType::PointerType DynamicCast(const NavigatorPtr& n)
        {
            return NavigatorType::DynamicCast(n);
        }

        template<typename NavigatorType>
        static typename NavigatorType::PointerType DynamicCastAndCheck(const NavigatorPtr& n)
        {
            return NavigatorType::DynamicCastAndCheck(n);
        }

    protected:
        static void CheckTypeNavigatorPtrForNull(const std::string&, const std::string&, const NavigatorPtr& data);
        static void CheckTypeNavigatorPtrForNull(const std::string&, const std::string&, const Path&, const NavigatorPtr& data);

    private:
        static const NavigatorFactoryPtr FACTORY;
    };
}
