/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Dict.h"


namespace armarx::aron::typenavigator
{
    // constructors
    DictNavigator::DictNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eDict, path),
        Navigator(type::Descriptor::eDict, path),
        type(new type::AronDict())
    {

    }

    DictNavigator::DictNavigator(const type::AronDictPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eDict, path),
        Navigator(type::Descriptor::eDict, path),
        type(o)
    {
        CheckAronPtrForNull("DictNavigator", "DictNavigator", getPath(), o);
        acceptedType = FromAronType(o->acceptedType);
    }

    NavigatorPtr DictNavigator::getAcceptedType() const
    {
        return acceptedType;
    }

    void DictNavigator::setAcceptedType(const NavigatorPtr& a)
    {
        type->acceptedType = a->getResult();
        acceptedType = a;
    }

    type::AronDictPtr DictNavigator::toAronDictPtr() const
    {
        CheckAronPtrForNull("DictNavigator", "getResult", getPath(), type->acceptedType);
        return type;
    }

    // static methods
    DictNavigatorPtr DictNavigator::DynamicCast(const NavigatorPtr& n)
    {
        return std::dynamic_pointer_cast<DictNavigator>(n);
    }

    DictNavigatorPtr DictNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckTypeNavigatorPtrForNull("DictNavigator", "DynamicCastAndCheck[Before]", n);
        DictNavigatorPtr casted = DictNavigator::DynamicCast(n);
        CheckTypeNavigatorPtrForNull("DictNavigator", "DynamicCastAndCheck[After]", n->getPath(), casted);
        return casted;
    }

    // virtual implementations
    std::vector<NavigatorPtr> DictNavigator::getChildren() const
    {
        return {acceptedType};
    }

    size_t DictNavigator::childrenSize() const
    {
        return 1;
    }

    type::AronTypePtr DictNavigator::getResult() const
    {
        CheckAronPtrForNull("DictNavigator", "getResult", getPath(), type->acceptedType);
        return type;
    }

    std::string DictNavigator::getName() const
    {
        return "AronDictType<" + acceptedType->getName() + ">";
    }
}

