/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>

namespace armarx::aron::typenavigator
{
    class IntEnumNavigator;
    typedef std::shared_ptr<IntEnumNavigator> IntEnumNavigatorPtr;

    class IntEnumNavigator :
        virtual public Navigator
    {
    public:
        using PointerType = IntEnumNavigatorPtr;

    public:
        // constructors
        IntEnumNavigator() = delete;
        IntEnumNavigator(const Path& path);
        IntEnumNavigator(const type::AronIntEnumPtr&, const Path& path);

        std::map<std::string, int> getAcceptedValues() const;
        std::string getValueName(int) const;
        int getValue(const std::string&) const;
        std::string getEnumName() const;

        void setEnumName(const std::string&);
        void addAcceptedValue(const std::string&, int);

        type::AronIntEnumPtr toAronIntEnumPtr() const;

        // static methods
        static IntEnumNavigatorPtr DynamicCast(const NavigatorPtr& n);
        static IntEnumNavigatorPtr DynamicCastAndCheck(const NavigatorPtr& n);

        // virtual implementations
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;
        virtual type::AronTypePtr getResult() const override;
        virtual std::string getName() const override;

    public:

    private:
        // members
        type::AronIntEnumPtr type;
    };
}
